<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\UniModul;

use Exception;
use mysqli;
use mysqli_result;

interface IDatabaseConnection {
	/**
	 * @param string $sql
	 * @return array<int, array<string, mixed>>
	 */
	function sqlQuery($sql);
	/**
	 * @param string $sql
	 * @return void
	 */
	function sqlExecute($sql);
	/** @return int */
	function getInsertId();
}


class MysqliConnection implements IDatabaseConnection {

	/** @var mysqli */
	protected $mysqli;

	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 */
	public function __construct($server, $login, $password, $dbname) {
		$this->connect($server, $login, $password, $dbname);
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
	}

	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 * @return void
	 */
	protected function connect ($server, $login, $password, $dbname) {
		if ($server === null) {
			return; // situace pro zjisteni jen config info ve factory
		} else if (preg_match('/^(.*):([0-9]+)$/', $server, $matches)) {
			$host = $matches[1]; $port = (int)$matches[2];
			$mysqli = mysqli_connect($host, $login, $password, $dbname, $port);
		} elseif (preg_match('#^.*:(/.*)$#', $server, $matches)) {
			$socket = $matches[1];
			$mysqli = mysqli_connect('', $login, $password, $dbname, intval(ini_get("mysqli.default_port")), $socket);
		} elseif (preg_match('#^(/.*)$#', $server, $matches)) {
			$socket = $matches[1];
			$mysqli = mysqli_connect('', $login, $password, $dbname, intval(ini_get("mysqli.default_port")), $socket);
		} else {
			$mysqli = mysqli_connect($server, $login, $password, $dbname);
		}
		if ($mysqli === false) {
			throw new Exception("Cannot connect to database: " . mysqli_connect_error());
		}
		$this->mysqli = $mysqli;
		$this->mysqli->set_charset("utf8");
	}


	public function sqlQuery($sql) {
		$res = $this->mysqli->query($sql);
		if ($res === false) {
			throw new Exception("Query Failed! SQL: $sql - Error: ".$this->mysqli->error);
		}
		$ar = array();
		if ($res !== true) {
			while($row = $res->fetch_array()) {
				$ar[] = $row;
			}
			$res->free();
		}
		return $ar;
	}


	public function sqlExecute($sql) {
		$res = $this->mysqli->query($sql);
		if (!$res) throw new Exception("Query Failed! SQL: $sql - Error: ".$this->mysqli->error);
		if ($res instanceof mysqli_result) $res->free();
	}

	function getInsertId() {
		$id = $this->mysqli->insert_id;
		if ($id != (int)$id) throw new Exception('Non int insert_id');
		return (int)$id;
	}

	/**
	 * @param scalar|null $text
	 * @return string
	 */
	function toSql($text) {
		if (is_null($text)) {
			return "null";
		} else {
			return "'".$this->mysqli->real_escape_string(strval($text))."'";
		}
	}

}

class MysqliConnectionLazy extends MysqliConnection {

	/** @var string|null */
	protected $server;
	/** @var string */
	protected $login;
	/** @var string */
	protected $password;
	/** @var string */
	protected $dbname;


	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 */
	public function __construct($server, $login, $password, $dbname) {
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
		$this->server = $server;
		$this->login = $login;
		$this->password = $password;
		$this->dbname = $dbname;
	}

	public function sqlQuery($sql) {
		$this->ensureConnected();
		return parent::sqlQuery($sql);
	}

	public function sqlExecute($sql) {
		$this->ensureConnected();
		parent::sqlExecute($sql);
	}

	public function getInsertId() {
		$this->ensureConnected();
		return parent::getInsertId();
	}

	public function toSql($text) {
		$this->ensureConnected();
		return parent::toSql($text);
	}

	/** @return void */
	protected function ensureConnected() {
		if ($this->mysqli === null) {
			$this->connect($this->server, $this->login, $this->password, $this->dbname);
		}
	}
}

class MysqliConnectionShared extends MysqliConnectionLazy {

	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 */
	public function __construct($server, $login, $password, $dbname) {
		global $MysqliConnectionShared_mysqli;
		if ($MysqliConnectionShared_mysqli != null) {
			$this->mysqli = $MysqliConnectionShared_mysqli;
		} else {
			parent::__construct($server, $login, $password, $dbname);
			$MysqliConnectionShared_mysqli = $this->mysqli;
		}
	}
}


/**
 * @param scalar|null $text
 * @return string
 */
function toSql($text) {
	return $GLOBALS['DatabaseConnection_toSql_object']->toSql($text);
}

