<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento2;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\Result\RawFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Controller\Result\Redirect as ResultRedirect;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Result\PageFactory;
use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Sales\Model\Order\Email\Sender\OrderSender as EmailOrderSender;
use Magento\Sales\Model\Order\Email\Sender\OrderCommentSender as EmailOrderCommentSender;
use Magento\Framework\Message\ManagerInterface as MessageManagerInterface;
use Magento\Framework\Locale\ResolverInterface as LocaleResolverInterface;
use Magento\Framework\App\DeploymentConfig;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Directory\Model\CurrencyFactory;
use Magento\Sales\Model\Order\Payment;
use Magento\Framework\Controller\Result\Redirect;

use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;
use Platiti\GPWebPayGpeNs\UniModul\RedirectAction;
use Platiti\GPWebPayGpeNs\UniModul\OrderStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItem;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItemType;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\Http;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use function Platiti\GPWebPayGpeNs\UniModul\ExceptionUniErr;
use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;

use Exception;
use Throwable;

require_once("UniAdapterHelper.php");


if (interface_exists(CsrfAwareActionInterface::class)) {
	require_once(dirname(__FILE__).'/ActionCsrfAllowed.php');
} else {
	abstract class ActionCsrfAllowed extends Action {
		abstract public function execute();
	}
}

class UniAdapterController extends ActionCsrfAllowed {
	/** @var string */
	protected $uniModulName; //predefinuje binder
	/** @var UniModul */
	protected $uniModul;
	/** @var string */
	protected $subMethod = '';
	/** @var ConfigSetting */
	protected $configSetting;
	/** @var UniAdapterHelper */
	public $helper;

	/** @var Context */
	protected $_context;
	/** @var Session */
	protected $_checkoutSession;
	/** @var Order */
	protected $_order;
	/** @var StoreManagerInterface */
	protected $_storeManager;
	/** @var PageFactory */
	protected $_resultPageFactory;
	/** @var RawFactory */
	protected $_resultRawFactory;
	/** @var ResultFactory */
	protected $_resultFactory;
	/** @var EmailOrderSender */
	protected $_emailOrderSender;
	/** @var EmailOrderCommentSender */
	protected $_emailOrderCommentSender;
	/** @var MessageManagerInterface */
	protected $_messageManagerInterface;

	public function __construct(
		Context $context,
		Order $order,
		Session $checkoutSession,
		StoreManagerInterface $storeManager,
		PageFactory $resultPageFactory,
		RawFactory $resultRawFactory,
		ResultFactory $resultFactory,
		EmailOrderSender $emailOrderSender,
		EmailOrderCommentSender $emailOrderCommentSender,
		MessageManagerInterface $messageManagerInterface,
		CurrencyFactory $currencyFactory,
		LocaleResolverInterface $localeResolver,
		ScopeConfigInterface $scopeConfig,
		DeploymentConfig $deploymentConfig
	) {
		$this->_checkoutSession = $checkoutSession;
		$this->_order = $order;
		$this->_storeManager = $storeManager;
		$this->_resultPageFactory = $resultPageFactory;
		$this->_resultRawFactory = $resultRawFactory;
		$this->_resultFactory = $resultFactory;
		$this->_emailOrderSender = $emailOrderSender;
		$this->_emailOrderCommentSender = $emailOrderCommentSender;
		$this->_messageManagerInterface = $messageManagerInterface;

		parent::__construct($context);

		$this->helper = new UniAdapterHelper($this->uniModulName, $this->subMethod, $storeManager, $currencyFactory, $localeResolver, $scopeConfig, $deploymentConfig);
		list($this->uniModul, $this->configSetting) = $this->helper->initUniModul([$this, 'funcProcessReplyStatus']);

	}

	/**
	 * @param string $haystack
	 * @param string $needle
	 *
	 * @return bool
	 */
	function startsWith($haystack, $needle) {
		$length = strlen($needle);
		return (substr($haystack, 0, $length) === $needle);
	}

	public function execute() {
		try {
			UniErr::begin();
			$action = strtolower(explode("binder/Index/Index" . strtolower($this->subMethod), $_SERVER['REQUEST_URI'])[1]);
			$action = str_replace("/", "", $action);

			if ($this->startsWith($action, "redirectaction")) {
				$result = $this->redirectAction();
			} elseif ($this->startsWith($action, "reply")) {
				$result = $this->replyAction();
				return UniErr::end($result);
			} elseif ($this->startsWith($action, "notify")) {
				$this->notifyAction();
				$result = $this->_resultRawFactory->create();
			} elseif ($this->startsWith($action, "callback")) {
				$this->callbackAction();
				$result = $this->_resultRawFactory->create();
			} else {
				throw new Exception('Unknown action: ' . $action);
			}
			return UniErr::end($result);
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}

	}


	/**
	 * @return ResultInterface
	 */
	public function redirectAction() {
		$subMethod = $this->subMethod;

		$order = $this->_checkoutSession->getLastRealOrder();
		$orderToPayInfo = $this->getOrderToPayInfo($order, $subMethod);

		try {
			// presmerovat na branu
			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);
		} catch (Exception $ex) {
			UniErr::logException($ex);
			$redirectAction = new RedirectAction();
			$orderReplyStatus = new OrderReplyStatus();
			$orderReplyStatus->orderStatus = OrderStatus::$invalidReply;
			$redirectAction->orderReplyStatus = $orderReplyStatus;
		} catch (Throwable $ex) {
			UniErr::logException($ex);
			$redirectAction = new RedirectAction();
			$orderReplyStatus = new OrderReplyStatus();
			$orderReplyStatus->orderStatus = OrderStatus::$invalidReply;
			$redirectAction->orderReplyStatus = $orderReplyStatus;
		}

		if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
			$frontend_redir = $redirectAction->redirectUrl == null && $redirectAction->redirectForm == null;
			$redirectResult = $this->processReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
			if ($frontend_redir) {
				return $redirectResult;
			}
		}

		if ($redirectAction->inlineForm != null) {
			$resultPage = $this->_resultPageFactory->create();
			$resultPage->initLayout();
			$block = $resultPage->getLayout()->getBlock("Index.Index." . strtolower($this->uniModulName . "-" . $this->subMethod));
			$block->setInlineContent($redirectAction->inlineForm); // @phpstan-ignore-line   - pouzuvame vlastni funkci definovoanou v nasem layoutu
			return $resultPage;
		} else if ($redirectAction->redirectForm != null) {
			$this->uniModul->webTools->formRedirect($redirectAction->redirectForm);
		} else if ($redirectAction->redirectUrl != null) {
			/** @var ResultRedirect $resultRedirect */
			$resultRedirect = $this->_resultFactory->create('redirect');
			$resultRedirect->setUrl($redirectAction->redirectUrl);
			return $resultRedirect;
		} else {
			throw new Exception('Invalid redirectAction');
		}
	}

	/**
	 * @return ResultInterface
	 */
	public function replyAction() {
		UniErr::begin();
		try {
			$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->helper->getLanguage());
		} catch (Exception $ex) {
			UniErr::logException($ex);
			$orderReplyStatus = new OrderReplyStatus();
			$orderReplyStatus->orderStatus = OrderStatus::$invalidReply;
		} catch (Throwable $ex) {
			UniErr::logException($ex);
			$orderReplyStatus = new OrderReplyStatus();
			$orderReplyStatus->orderStatus = OrderStatus::$invalidReply;
		}
		$r = $this->processReplyStatus($orderReplyStatus, true);
		return UniErr::end($r);
	}

	/**
	 * @return void
	 */
	public function notifyAction() {
		UniErr::begin();
		$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
		$this->processReplyStatus($orderReplyStatus, false);
		UniErr::end();
	}

	/**
	 * @return void
	 */
	public function callbackAction() {
		UniErr::begin();
		parse_str($_SERVER['QUERY_STRING'], $cbArgs);
		/** @var array<string, string> $cbArgs */ // assuming unique GET parameter names
		$this->uniModul->processCallbackRequest(Http::get('_callbackName'), $cbArgs);
		UniErr::end();
	}

	/**
	 * @param Order $order
	 * @param ?string $subMethod
	 *
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo($order, $subMethod) {

		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		$orderToPayInfo->shopPairingInfo = $order->getIncrementId();
		$orderToPayInfo->shopOrderNumber = $order->getIncrementId();

		$orderToPayInfo->amount = (float) $order->getGrandTotal();
		$orderToPayInfo->currency = $order->getOrderCurrencyCode() ?: $this->throwExc(new Exception("Currency not found"));
		$orderToPayInfo->uniAdapterData = array('order_total'=>$order->getBaseGrandTotal());

		$store = $this->_storeManager->getStore();
		assert($store instanceof Store);
		$description = $store->getFrontendName();
		$products = array();
		$cartItems = [];
		foreach ($order->getAllVisibleItems() as $item) {
			$products[] = $item->getName();

			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $item->getName();
			$cartItem->quantity = $item->getQtyToShip();
			$cartItem->unitPrice = (float) $item->getPriceInclTax();
			$cartItem->taxRate = round($item->getTaxPercent());
			$cartItems[] = $cartItem;
		}
		$orderToPayInfo->description = $description;

		$cartItem = new UniCartItem();
		$cartItem->type = UniCartItemType::delivery;
		$cartItem->name = 'shipping / doprava';
		$cartItem->quantity = 1;
		$cartItem->unitPrice = (float) $order->getShippingInclTax();
		$cartItem->taxRate = $order->getShippingAmount() == 0 ? 0 : round(100 * $order->getShippingTaxAmount() / $order->getShippingAmount());
		$cartItems[] = $cartItem;

		$discountAmount = $order->getDiscountAmount();
		if ($discountAmount != 0) {
			// secteni pres sazby
			$rozdeleni = array_reduce($cartItems, function ($a, $ci) {
				$a[(string)$ci->taxRate] = $ci->unitPrice * $ci->quantity + (isset($a[$ci->taxRate]) ? $a[$ci->taxRate] : 0);
				return $a;
			}, array());


			// rovnomerne rozpocteni slevy na pasma
			$totalBeforeDiscount = $orderToPayInfo->amount - $discountAmount;
			foreach ($rozdeleni as $tax => $amt) {
				$cartItem = new UniCartItem();
				$cartItem->type = UniCartItemType::discount;
				$cartItem->name = 'discount / sleva';
				$cartItem->quantity = 1;
				$cartItem->unitPrice = (float) $discountAmount * $amt / $totalBeforeDiscount;
				$cartItem->taxRate = $tax;
				$cartItems[] = $cartItem;
			}
		}
		$orderToPayInfo->cartItems = $cartItems;

		$orderToPayInfo->customerData = $this->helper->getCustomerData($order);

		$code = ucfirst(strtolower($this->uniModulName)) . 'binder';
		$baseUrl = $store->getBaseUrl();
		$orderToPayInfo->notifyUrl = $baseUrl . $code . '/Index/Index/notify';
		$orderToPayInfo->replyUrl = $baseUrl . $code . '/Index/Index/reply';
		if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
			$orderToPayInfo->uniModulDirUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB) . "UniModul/".ucfirst(strtolower($this->uniModulName))."binder/";
		} else {
			$orderToPayInfo->uniModulDirUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB) . "UniModul/UniModul/";
		}
		$orderToPayInfo->language = $this->helper->getLanguage();
		$orderToPayInfo->currencyRates = $this->helper->getCurrencyRates();
		//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty
		return $orderToPayInfo;
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 *
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
		$this->processReplyStatus($orderReplyStatus, false);
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param bool $frontend_redir
	 *
	 * @return ResultInterface
	 */
	function processReplyStatus($orderReplyStatus, $frontend_redir) {

		$cfg = $this->configSetting->configData;

		$order = $this->_order;

		switch ($orderReplyStatus->orderStatus)
		{
			case OrderStatus::$successful:
				$newState = $order::STATE_PROCESSING;
				$newStatus = $cfg['orderStatusSuccessfull'];
				$guiOk = true;
				break;
			case OrderStatus::$pending:
				$newState = $order::STATE_PENDING_PAYMENT;
				$newStatus = $cfg['orderStatusPending'];
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
				$newState = $order::STATE_CANCELED;
				$newStatus = $cfg['orderStatusFailed'];
				$guiOk = false;
				break;
			case OrderStatus::$invalidReply;
				$newStatus = null;
				$newState = null;
				break;
			default:
				$this->uniModul->logger->writeLog("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
				$newStatus = null;
				$newState = null;
		}

		if ($newStatus == null) {
			$this->uniModul->logger->writeLog("ADAPTER: invalidReply => no effect");
		} else {
			BeginSynchronized();

			$order->loadByIncrementId($orderReplyStatus->shopPairingInfo);

			if (!$order->getId()) {
				// TODO  nacteni se nepovedlo...
				EndSynchronized();
				throw new Exception("nacteni orderu se nepovedlo");
			}
			if ($order->getState() == $newState || !in_array($order->getState(), array($order::STATE_NEW, $order::STATE_PENDING_PAYMENT, $order::STATE_CANCELED))) {
				$this->uniModul->logger->writeLog("ADAPTER: Not updating shop order number ".$order->getIncrementId().", old order state ".$order->getState().", new order state ".$newState);
			} else {

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->helper->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->helper->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				/** @var Payment|null */
				$payment = $order->getPayment();
				if ($payment === null) {
					EndSynchronized();
					throw new Exception("Failed to load payment!");
				}
				$payment->setTransactionId($orderReplyStatus->gwOrderNumber.'');
				$payment->setIsTransactionClosed(false);

				if ($newState == $order::STATE_PROCESSING) {

					$payment->registerCaptureNotification($orderReplyStatus->uniAdapterData['order_total']);
					$payment->setPreparedMessage($comment); // @phpstan-ignore-line

					$invoice = $payment->getCreatedInvoice(); // @phpstan-ignore-line
					if ($invoice && !$order->getEmailSent()) {
						$this->uniModul->logger->writeLog("ADAPTER: Sending email - OrderSender");
						$this->_emailOrderSender->send($order);
					}

				} else if ($newState == $order::STATE_CANCELED) {
					$order->registerCancellation($comment);
				}

				$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number ".$order->getIncrementId().", old order state ".$order->getState().", new order state ".$newState. ", new order status: " . $newStatus);
				$order->setState($newState);
				$history = $order->addCommentToStatusHistory($comment, $newStatus, true);
				$history->setIsVisibleOnFront(1);
				$history->setIsCustomerNotified(1);
				$order->save();
				$this->uniModul->logger->writeLog("ADAPTER: Sending email - OrderCommentSender");
				$this->_emailOrderCommentSender->send($order, true, $comment);

			}
			EndSynchronized();
		}



		if ($frontend_redir) {
			$resultFactory = $this->_resultFactory;
			/** @var Redirect */
			$resultRedirect = $resultFactory->create('redirect');
			if ($orderReplyStatus->orderStatus == OrderStatus::$pending || $orderReplyStatus->orderStatus == OrderStatus::$successful) {
				//Mage::getSingleton('core/session')->addNotice('Order OK.');
				$this->_checkoutSession->getQuote()->setData('UniModulSuccessHtml', $orderReplyStatus->successHtml);
				$resultRedirect->setPath('checkout/onepage/success', array('_secure' => true));
			} else {
				$this->_checkoutSession->restoreQuote();
				$failMsg = __('Payment failed. Please try again.') . " " . $orderReplyStatus->resultText;
				$this->_messageManagerInterface->addError($failMsg);
				$resultRedirect->setPath('checkout/cart');
			}
			return $resultRedirect;
		}
		return $this->_resultRawFactory->create();
	}

	/**
	 * @return string
	 */
	public function getSuccessHtml() {
		return (string)$this->_checkoutSession->getQuote()->getData('UniModulSuccessHtml');
	}


	/**
	 * @param Exception $e
	 * @return never
	 */
	protected function throwExc($e) {
		throw $e;
	}
}
