<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento2;

use Magento\Framework\Config\ConfigOptionsListConstants;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\UrlInterface;
use Magento\Quote\Model\Quote;
use Magento\Sales\Model\Order;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\Filesystem\DirectoryList;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Directory\Model\CurrencyFactory;
use Magento\Framework\Locale\ResolverInterface as LocaleResolverInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\DeploymentConfig;
use Magento\Store\Model\Store;

use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\ConfigField;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;
use Platiti\GPWebPayGpeNs\UniModul\ConfigFieldType;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;

use function Platiti\GPWebPayGpeNs\UniModul\create_initialize_object;

if (!defined('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath')) define('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath',
	ObjectManager::getInstance()->get(DirectoryList::class)->getPath('var') . '/UniModul');


require_once("UniModul.php");

class UniAdapterHelper {

	/** @var string */
	protected $uniModulName;
	/** @var string */
	protected $subMethod;

	/** @var StoreManagerInterface */
	protected $storeManager;
	/** @var CurrencyFactory */
	protected $currencyFactory;
	/** @var LocaleResolverInterface */
	protected $localeResolver;
	/** @var ScopeConfigInterface */
	protected $scopeConfig;
	/** @var DeploymentConfig */
	protected $deploymentConfig;

	/**
	 * @param string $uniModulName
	 * @param string $subMethod
	 * @param StoreManagerInterface $storeManager
	 * @param CurrencyFactory $currencyFactory
	 * @param LocaleResolverInterface $localeResolver
	 * @param ScopeConfigInterface $scopeConfig
	 * @param DeploymentConfig $deploymentConfig
	 * @return void
	 */
	public function __construct($uniModulName, $subMethod, $storeManager, $currencyFactory, $localeResolver, $scopeConfig, $deploymentConfig) {
		$this->uniModulName = $uniModulName;
		$this->subMethod = $subMethod;

		$this->storeManager = $storeManager;
		$this->currencyFactory = $currencyFactory;
		$this->localeResolver = $localeResolver;
		$this->scopeConfig = $scopeConfig;
		$this->deploymentConfig = $deploymentConfig;
	}

	/**
	 * @return array<string, float>
	 */
	public function getCurrencyRates() {
		$currencyRates = array();
		/** @var Store */
		$store = $this->storeManager->getStore();
		foreach ($store->getAvailableCurrencyCodes(false) as $availableCurrencyCode) {
			/** @var string $availableCurrencyCode */
			$currency = $this->currencyFactory->create()->load($availableCurrencyCode);
			$currencyRates[$availableCurrencyCode] = (float)$store->getBaseCurrency()->getRate($availableCurrencyCode);
		}
		return $currencyRates;
	}


	/** @return string  */
	public function getLanguage() {
		return substr($this->localeResolver->getLocale(), 0, 2);
	}


	/**
	 * @param Quote | Order $order
	 * @return CustomerData
	 */
	public function getCustomerData($order) {
		$billing = $order->getBillingAddress() ?: $order->getShippingAddress();
		$customerData = new CustomerData();
		$customerData->first_name = $order->getCustomerFirstname() ?: ($billing === null ? '' : $billing->getFirstname()) ?: '';
		$customerData->last_name = $order->getCustomerLastname() ?: ($billing === null ? '' : $billing->getLastname()) ?: '';
		$customerData->email = $order->getCustomerEmail() ?: ($billing === null ? '' : $billing->getEmail()) ?: '';
		if ($billing !== null) {
			$customerData->street = implode(' ', $billing->getStreet());
			$customerData->city = $billing->getCity();
			$customerData->post_code = $billing->getPostcode();
			$customerData->country = $billing->getCountryId();
			$customerData->phone = $billing->getTelephone();
			$customerData->identifier = $billing->getCustomerId() !== null ? (string)$billing->getCustomerId() : null;  // pro guesta je to null
		}
		return $customerData;
	}


	/**
	 * @param ?callable(OrderReplyStatus) : void $funcProcessReplyStatus
	 * @return array{0: UniModul, 1: ConfigSetting}
	 */
	public function initUniModul($funcProcessReplyStatus = null) {
		$uniFact = new UniModulFactory();
		$configInfo = $uniFact->getConfigInfo($this->uniModulName, self::getLanguage(), $this->subMethod, 'Magento2');
		$configSetting = self::getConfigDataUni($configInfo, $funcProcessReplyStatus);
		$uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
		return [$uniModul, $configSetting];
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param ?callable(OrderReplyStatus) : void $funcProcessReplyStatus
	 * @return ConfigSetting
	 */
	protected function getConfigDataUni($configInfo, $funcProcessReplyStatus) {

		// doplnime failed stav, i kdyz ho modul nevyzaduje, protoze Magento vzdy vytvori order
		if (array_filter($configInfo->configFields, array(self::class, "getConfigDataUni_filter")) === array()) {
			$configField = new ConfigField();
			$configField->name = 'orderStatusFailed';
			$configField->type = ConfigFieldType::$orderStatus;
			$configInfo->configFields[] = $configField;
		}

		$configData = array();
		$path = "payment/" . strtolower($this->uniModulName) . "binder/";

		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = $this->scopeConfig->getValue($path . $configField->name, ScopeInterface::SCOPE_STORE);
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->mysql_server = $this->deploymentConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/host' );
		$uniModulConfig->mysql_dbname = $this->deploymentConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/dbname' );
		$uniModulConfig->mysql_login = $this->deploymentConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/username');
		$uniModulConfig->mysql_password = $this->deploymentConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/password');

		/** @var Store */
		$store = $this->storeManager->getStore();
		if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
			$uniModulConfig->uniModulDirUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB) . "UniModul/".ucfirst(strtolower($this->uniModulName))."binder/";
		} else {
			$uniModulConfig->uniModulDirUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB) . "UniModul/UniModul/";
		}
		$uniModulConfig->funcGetCallbackUrl = [$this, 'getCallbackUrl'];
		if ($funcProcessReplyStatus !== null) $uniModulConfig->funcProcessReplyStatus = $funcProcessReplyStatus;
		$uniModulConfig->legacyUniModulVarPaths = [BP . '/UniModul', BP . '/lib/UniModul'];

		$uniModulConfig->adapterName = 'Magento2';
		$cfgs = create_initialize_object(ConfigSetting::class, array('configData' => $configData, 'uniModulConfig' => $uniModulConfig));

		return $cfgs;

	}

	/**
	 * @param ConfigField $e
	 * @return bool
	 */
	private function getConfigDataUni_filter($e) {
		return $e->name == 'orderStatusFailed';
	}


	/**
	 * @param string $callbackName
	 * @param mixed $arguments
	 * @return string
	 */
	public function getCallbackUrl($callbackName, $arguments) {
		$code = ucfirst(strtolower( $this->uniModulName)) . 'binder';
		/** @var Store */
		$store = $this->storeManager->getStore();
		$baseUrl = $store->getBaseUrl();
		return $baseUrl.$code.'/Index/Index/callback' . "?_unimodul=" . $this->uniModulName . "&_callbackName=" . urlencode($callbackName) . "&" . http_build_query($arguments);;
	}
}