<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento2;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data as PaymentHelper;
use Magento\Directory\Helper\Data as DirectoryHelper;
use Magento\Payment\Model\Method\AbstractMethod;
use Magento\Payment\Model\Method\Logger;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\Quote;
use Magento\Checkout\Model\Session;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Locale\ResolverInterface as LocaleResolverInterface;
use Magento\Framework\App\DeploymentConfig;
use Magento\Directory\Model\CurrencyFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Store\Model\Store;

use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\PrePayGWInfo;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use function Platiti\GPWebPayGpeNs\UniModul\ExceptionUniErr;

use Exception;
use Throwable;

require_once("UniAdapterHelper.php");

class UniAdapterModel extends AbstractMethod {
	/** @var string */
	protected $uniModulName; //predefinuje binder
	/** @var UniModul */
	public $uniModul;
	/** @var string */
	protected $subMethod;
	/** @var ConfigSetting */
	protected $configSetting;
	/** @var UniAdapterHelper */
	protected $helper;

	/** @var bool */
	protected $_isOffline = true;

	protected $_code = '';
	//protected $_shouldInactivateQuote = false;   //nefunguje pro zachovani kosiku
	protected $_isGateway = true;
	protected $_isInitializeNeeded = true;
	protected $_formBlockType = 'platiti/form';
	protected $_canCapture = true;

	/** @var Session */
	protected $_checkoutSession;
	/** @var StoreManagerInterface */
	protected $_storeManager;
	/** @var Registry */
	protected $_registry;
	/** @var ScopeConfigInterface */
	protected $_scopeConfig;

	/**
	 * @param mixed[] $data
	 *
	 */
	public function __construct(
		Context $context,
		Registry $registry,
		ExtensionAttributesFactory $extensionFactory,
		AttributeValueFactory $customAttributeFactory,
		PaymentHelper $paymentData,
		ScopeConfigInterface $scopeConfig,
		Logger $logger,
		Session $session,
		StoreManagerInterface $storeManager,
		CurrencyFactory $currencyFactory,
		LocaleResolverInterface $localeResolver,
		DeploymentConfig $deploymentConfig,
		AbstractResource $resource = null,
		AbstractDb $resourceCollection = null,
		array $data = [],
		DirectoryHelper $directory = null
	) {
		parent::__construct(
			$context,
			$registry,
			$extensionFactory,
			$customAttributeFactory,
			$paymentData,
			$scopeConfig,
			$logger,
			$resource,
			$resourceCollection,
			$data,
			$directory
		);
		$this->_checkoutSession = $session;
		$this->_storeManager = $storeManager;
		$this->_registry = $registry;
		$this->_scopeConfig = $scopeConfig;
		$this->helper = new UniAdapterHelper($this->uniModulName, $this->subMethod, $storeManager, $currencyFactory, $localeResolver, $scopeConfig, $deploymentConfig);
		list($this->uniModul, $this->configSetting) = $this->helper->initUniModul();
	}


	// jinak se nevola initialize
	/** @return mixed  */
	public function getConfigPaymentAction() {
		UniErr::begin();
		$paymentAction = $this->getConfigData('payment_action');
		return UniErr::end(empty($paymentAction) ? true : $paymentAction);
	}

	/**
	 * @return PrePayGWInfo
	 */
	public function getPrePayGwInfo(CartInterface $quote = null) {
		if ($quote == null) {
			$quote = $this->_checkoutSession->getQuote();
		}


		$orderToPayInfo = $this->getOrderToPayInfo($quote, $this->subMethod);
		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
		return $prePayGWInfo;
	}

	/**
	 * @param CartInterface|null $quote
	 * @return bool
	 */
	public function isAvailable(CartInterface $quote = null) {
		try {
			UniErr::begin();
			//u jednoho eshopu je stává že magento předá null místo quote

			$prePayGWInfo = $this->getPrePayGwInfo($quote);
			return UniErr::end($prePayGWInfo->isPossible);
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}


	/**
	 * @param CartInterface $quote
	 * @param string $subMethod
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo($quote, $subMethod) {
		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $subMethod;
		$billing = $quote->getBillingAddress();

		if (!($quote instanceof Quote)) throw new Exception("Invalid quote");
		$orderToPayInfo->amount = (float) $quote->getGrandTotal();
		$orderToPayInfo->currency = $quote->getQuoteCurrencyCode();

		$orderToPayInfo->customerData = $this->helper->getCustomerData($quote);

		$orderToPayInfo->language = $this->helper->getLanguage();
		$orderToPayInfo->currencyRates = $this->helper->getCurrencyRates();
		return $orderToPayInfo;
	}

	/** @return string  */
	function getMiniLogo() {
		return $this->uniModul->getMiniLogoSpan();
	}

	/** @return string  */
	function getImage() {
		return $this->uniModul->getModulSubMethodLogoImage2($this->uniModul->subMethod);
	}


	/** @return ?string */
	public function getInstallmentHtml() {
		try {
			UniErr::begin();
			$path = "payment/" . strtolower($this->uniModulName) . "binder/";
			if(!$this->_scopeConfig->getValue($path . "active")){
				return(UniErr::end(''));
			}
			$currentProduct = $this->_registry->registry('current_product');
			$price = $currentProduct->getFinalPrice();
			/** @var Store */
			$store = $this->_storeManager->getStore();
			$product_price = $store->getBaseCurrency()->convert($price, $store->getCurrentCurrency());
			$current_currency = $store->getCurrentCurrency()->getCurrencyCode();

			$baseUrl = $store->getBaseUrl(UrlInterface::URL_TYPE_WEB);
			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml($baseUrl, $current_currency, $product_price);

			return UniErr::end($embedHtml);
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}


}
