<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\Magento2;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\App\DeploymentConfig;

use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\UniLogger;
use Platiti\GPWebPayGpeNs\UniModul\MysqliConnectionShared;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use function Platiti\GPWebPayGpeNs\UniModul\ExceptionUniErr;

use Exception;
use Throwable;

require_once("UniAdapterHelper.php");

/**
 * install function runs on command 'bin/magento setup:upgrade'
 */

class UniAdapterSetupRecurring implements InstallSchemaInterface
{
	/**
	 * @var DeploymentConfig
	 */
	protected $deploymentConfig;

	/**
	 * @param DeploymentConfig $deploymentConfig
	 */
	public function __construct(
		DeploymentConfig $deploymentConfig
	) {
		$this->deploymentConfig = $deploymentConfig;
	}

	/**
	 * {@inheritdoc}
	 */
	public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
	{
		try {
			UniErr::begin();
			$installer = $setup;
			$installer->startSetup();

			$logger = UniLogger::getLogger();
			$dbConfig = $this->deploymentConfig->get('db/connection/default');
			$dbConnection = new MysqliConnectionShared($dbConfig["host"], $dbConfig["username"], $dbConfig["password"], $dbConfig["dbname"]);
			UniModul::moduleInstallDatabaseStatic($dbConnection, $logger);
			UniModul::moduleInstallVarDirStatic($logger);

			$sourceDir = __DIR__;
			$binderName = basename(dirname(__DIR__));
			$linkDir = BP . "/pub/UniModul/";
			if (!is_dir($linkDir)) {
				mkdir($linkDir, 0755, true);
			}
			if ($binderName !== "lib") {
				if (!is_link($linkDir . $binderName)) {
					symlink($sourceDir, $linkDir . $binderName);
				}
			} else {
				if (!is_link($linkDir . "UniModul")) {
					symlink($sourceDir, $linkDir . "UniModul");
				}
			}


			$installer->endSetup();
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}
}
